/// A base class for all Data Transfer Objects (DTOs).
///
/// This class provides a common structure for all DTOs, including an ID, active status, deleted status, and timestamps.
/// It also provides methods to convert the DTO to and from a JSON object.
///
/// All DTOs should inherit from this class and add any additional properties that they need.
class BaseDTO {
  /// The ID of the DTO.
  String? id;

  /// Whether the DTO is active.
  bool? isActive;

  /// Whether the DTO has been deleted.
  bool? isDeleted;

  /// The time when the DTO was created.
  String? createdAt;

  /// The time when the DTO was last updated.
  String? updatedAt;

  /// Creates a new instance of [BaseDTO].
  ///
  /// The parameters can be used to specify the initial values for the DTO's properties.
  BaseDTO({
    this.id,
    this.isActive,
    this.isDeleted,
    this.createdAt,
    this.updatedAt,
  });

  /// Creates a new instance of [BaseDTO] from a JSON object.
  ///
  /// This method extracts the properties from the [json] parameter and uses them to initialize a new instance of [BaseDTO].
  BaseDTO.fromJson(Map<String, dynamic> json) {
    id = json['id'];
    isActive = json['isActive'];
    isDeleted = json['isDeleted'];
    createdAt = json['createdAt'];
    updatedAt = json['updatedAt'];
  }

  /// Converts the [BaseDTO] to a JSON object.
  ///
  /// This method can be used when making HTTP requests to convert the DTO to a format that can be sent in the request.
  Map<String, dynamic> toJson() {
    final Map<String, dynamic> data = <String, dynamic>{};
    data["id"] = id;
    data["isActive"] = isActive;
    data["isDeleted"] = isDeleted;
    data["createdAt"] = createdAt;
    data["updatedAt"] = updatedAt;
    return data;
  }
}
