import 'package:parodia_admin/core/dto/base.dto.dart';
import 'package:parodia_admin/core/model/response.model.dart';
import 'package:parodia_admin/core/model/validation_error.model.dart';

/// A class for handling response items.
///
/// This class represents a response item, with a status, message, error, and required validation errors.
/// It also provides methods to convert the response item to and from a JSON object.
class ResponseItemDTO<T> {
  /// The item of the response.
  T? item;

  /// The function to convert the item from a JSON object.
  final Function(Map<String, dynamic>)? fromJson;

  /// The status of the response.
  int? _status;
  int? get status => _status;

  /// Whether the response has an error.
  bool _hasError = false;
  bool get hasError => _hasError;

  /// The message of the response.
  String? _message;
  String? get message => _message;

  /// The error of the response.
  dynamic _error;
  dynamic get error => _error;

  /// The required validation errors of the response.
  List<ValidationErrorModel>? _required;
  List<ValidationErrorModel>? get required => _required;

  /// Creates a new instance of [ResponseItemDTO].
  ///
  /// The [response] parameter is used to initialize the response item.
  /// The [fromJson] parameter is used to convert the item from a JSON object.
  /// The [isSingleData] parameter is used to specify whether the response data is a single item or a list of items.
  ResponseItemDTO({
    required ResponseModel response,
    this.fromJson,
    bool isSingleData = false,
  }) {
    _message = response.message;
    _status = response.statusCode;

    if (response.statusCode! >= 400) {
      _hasError = true;
      _error = response.data ?? response.message;
      if (response.required != null && response.required!.isNotEmpty) {
        _required = response.required;
      }
      return;
    }
    if (response.data == null) {
      return;
    }

    if (isSingleData) {
      item = response.data;
    } else {
      if (fromJson != null) {
        item = fromJson!(response.data);
      }
    }
  }

  /// Converts the [ResponseItemDTO] to a JSON object.
  ///
  /// This method can be used when making HTTP requests to convert the response item to a format that can be sent in the request.
  Map<String, dynamic> toJson() {
    Map<String, dynamic> data = <String, dynamic>{};
    data["message"] = message;
    data["status"] = status;
    if (item != null) {
      data["item"] = (item! as BaseDTO).toJson();
    }
    if (hasError) {
      data["hasError"] = hasError;
      data["error"] = error;
      if (required != null && required!.isNotEmpty) {
        data["required"] = required!.map((e) => e.toJson());
      }
    }
    return data;
  }
}
