import 'package:parodia_admin/core/dto/base.dto.dart';
import 'package:parodia_admin/core/model/response.model.dart';

/// A class for handling response lists.
///
/// This class represents a response list, with a list of items, count, status, message, and error.
/// It also provides methods to convert the response list to and from a JSON object.
class ResponseListDTO<T extends BaseDTO> {
  /// The items of the response list.
  List<T> items = [];

  /// The count of the items in the response list.
  int? count;

  /// The function to convert the items from a JSON object.
  final Function(Map<String, dynamic>) fromJson;

  /// The status of the response.
  int? _status;
  int? get status => _status;

  /// Whether the response has an error.
  bool _hasError = false;
  bool get hasError => _hasError;

  /// The message of the response.
  String? _message;
  String? get message => _message;

  /// The error of the response.
  dynamic _error;
  dynamic get error => _error;

  /// Creates a new instance of [ResponseListDTO].
  ///
  /// The [response] parameter is used to initialize the response list.
  /// The [fromJson] parameter is used to convert the items from a JSON object.
  ResponseListDTO({
    required ResponseModel response,
    required this.fromJson,
  }) {
    _message = response.message;
    _status = response.statusCode;

    if (response.statusCode! >= 400) {
      _hasError = true;
      _error = response.data ?? response.message;
      return;
    }
    if (response.data == null) {
      return;
    }

    if (response.data is List) {
      items =
          (response.data as List).map((x) => fromJson(x)).toList().cast<T>();
      count = items.length;
      return;
    } else {
      items = (response.data["result"] as List)
          .map((x) => fromJson(x))
          .toList()
          .cast<T>();
      count = response.data["count"] ?? items.length ?? 0;
    }
  }

  /// Converts the [ResponseListDTO] to a JSON object.
  ///
  /// This method can be used when making HTTP requests to convert the response list to a format that can be sent in the request.
  Map<String, dynamic> toJson() {
    Map<String, dynamic> data = <String, dynamic>{};
    data["message"] = message;
    data["status"] = status;

    if (items.isNotEmpty) {
      data["items"] = items.map((e) => e.toJson()).toList();
    }
    if (hasError) {
      data["hasError"] = hasError;
      data["error"] = error;
    }

    return data;
  }
}
