import 'package:dio/dio.dart';
import 'package:parodia_admin/core/enums.dart';
import 'package:parodia_admin/core/model/body_data.model.dart';
import 'package:parodia_admin/core/model/query.model.dart';
import 'package:parodia_admin/core/model/response.model.dart';

// A class for making HTTP requests.
///
/// This class provides a common structure for making HTTP requests, including setting headers and handling different types of requests.
/// It uses the Dio package to make the requests.
/// Provides methods to make GET, POST, PUT, and DELETE requests to a specified path.
/// It returns a [ResponseModel] object, which can be used to check the status code and message of the response, and to access the data.
class ParodiaHTTP {
  /// The base options for the Dio instance.
  final BaseOptions _options = BaseOptions(
    followRedirects: false,
    validateStatus: (status) {
      return true;
    },
  );

  /// The headers to be included in the requests.
  final Map<String, dynamic> _headers = {};

  /// The base URL for the requests.
  String _baseUrl = "";
  set setBaseUrl(String baseUrl) {
    _baseUrl = baseUrl;
    _options.baseUrl = _baseUrl;
  }

  /// The store key to be included in the headers.
  String _storeKey = "";
  set setStoreKey(String storeKey) {
    _storeKey = storeKey;
    _headers["Store"] = _storeKey;
    _options.headers = _headers;
  }

  /// The authentication token to be included in the headers.
  String? _authToken;
  String? get authToken => _authToken;
  set setAuthToken(String? authToken) {
    _authToken = authToken;
    _headers["Authorization"] = "Bearer $_authToken";
    _options.headers = _headers;
  }

  /// The currency to be included in the headers.
  String? _currency;
  String? get getCurrency => _currency;
  set setCurrency(String? currency) {
    _currency = currency;
    _headers["Currency"] = currency;
    _options.headers = _headers;
  }

  /// The Dio instance used to make the requests.
  Dio? _dio;
  set setDio(Dio dio) {
    _dio = dio;
  }

  /// Creates a new instance of [ParodiaHTTP].
  ///
  /// The parameters can be used to specify the initial values for the base URL, store key, authentication token, and currency.
  ParodiaHTTP({
    required String baseUrl,
    required String storeKey,
    String? authToken,
    String? currency,
  }) {
    _dio = Dio();
    _baseUrl = baseUrl;
    _storeKey = storeKey;
    _authToken = authToken;
    _currency = currency;

    _headers["Store"] = _storeKey;

    if (authToken != null) {
      _headers["Authorization"] = "Bearer $_authToken";
    }

    if (currency != null) {
      _headers["Currency"] = currency;
    }

    _options.baseUrl = _baseUrl;
    _options.headers = _headers;
    _dio!.options = _options;
  }

  /// Makes a request.
  ///
  /// This method makes a request of the specified type to the specified path.
  /// It can optionally include a parameter, a query model, a body data model, and a content type.
  /// It returns a [ResponseModel] that represents the response from the server.
  Future<ResponseModel> request({
    required RequestType requestType,
    required String path,
    String? param,
    QueryModel? query,
    BodyDataModel? data,
    ContentType? contentType,
  }) async {
    switch (contentType) {
      case ContentType.json:
        _dio!.options.contentType = "application/json";
        break;
      case ContentType.multipart:
        _dio!.options.contentType = "multipart/form-data";
        break;
      default:
        _dio!.options.contentType = "application/json";
        break;
    }

    try {
      Response? response;
      Map<String, dynamic>? filteredData;
      FormData? formData = await data?.toFormData();
      if (data != null && formData == null) {
        final postData = data.toPostJson();
        final putData = data.toPutJson();
        if (postData != null) {
          postData.removeWhere((key, value) => value == null || value == "");
        }
        if (putData != null) {
          putData.removeWhere((key, value) => value == null || value == "");
        }
        if (requestType == RequestType.post) {
          filteredData = postData;
        } else if (requestType == RequestType.put) {
          filteredData = putData;
        }
      }
      Map<String, dynamic>? filteredQuery;
      if (query != null) {
        final queryData = query.toJson();
        queryData.removeWhere((key, value) => value == null || value == "");
        filteredQuery = queryData;
      }
      switch (requestType) {
        case RequestType.get:
          response = await _dio!.get(
            "$path/${param ?? ""}",
            queryParameters: filteredQuery,
          );
          break;
        case RequestType.post:
          response = await _dio!.post(
            "$path/${param ?? ""}",
            data: formData ?? filteredData,
            queryParameters: filteredQuery,
          );
          break;
        case RequestType.put:
          response = await _dio!.put(
            "$path/${param ?? ""}",
            data: formData ?? filteredData,
            queryParameters: filteredQuery,
          );
          break;
        case RequestType.delete:
          response = await _dio!.delete(
            "$path/${param ?? ""}",
            queryParameters: filteredQuery,
          );
          break;
      }
      return ResponseModel.fromJson(response.data);
    } catch (e) {
      if (e is DioException) {
        return ResponseModel(
          statusCode: 999,
          message: e.message,
        );
      } else {
        return ResponseModel(
          statusCode: 999,
          message: "Unexpected error occured.",
          data: e,
        );
      }
    }
  }
}
