import 'dart:io';

import 'package:dio/dio.dart';

/// A base class for all body data models.
///
/// This class provides a common structure for all body data models, including methods to convert the model to JSON and FormData.
/// It is designed to be used with HTTP POST and PUT requests, where the body of the request needs to be sent as JSON or FormData.
///
/// All body data models should inherit from this class and implement the `toPostJson`, `toPutJson`, and `toFormData` methods.
abstract class BodyDataModel {
  /// Converts the body data model to a JSON object for a POST request.
  ///
  /// This method should be implemented by subclasses to convert their specific data to a JSON object.
  /// It should only include properties that need to be sent in a POST request.
  Map<String, dynamic>? toPostJson();

  /// Converts the body data model to a JSON object for a PUT request.
  ///
  /// This method should be implemented by subclasses to convert their specific data to a JSON object.
  /// It should only include properties that need to be sent in a PUT request.
  Map<String, dynamic>? toPutJson();

  /// Converts the body data model to a FormData object.
  ///
  /// This method should be implemented by subclasses to convert their specific data to a FormData object.
  /// It is designed to be used with requests that need to send data as FormData, such as file uploads.
  Future<FormData?>? toFormData();
}

class LoginBody extends BodyDataModel {
  String? email;
  String? password;

  LoginBody({
    this.email,
    this.password,
  });

  @override
  Map<String, dynamic>? toPostJson() {
    Map<String, dynamic> data = <String, dynamic>{};
    data["email"] = email;
    data["password"] = password;
    return data;
  }

  @override
  Map<String, dynamic>? toPutJson() {
    return null;
  }

  @override
  Future<FormData?>? toFormData() {
    return null;
  }
}

class ForgotPasswordBody extends BodyDataModel {
  String? code;
  String? email;
  String? password;

  ForgotPasswordBody({
    this.code,
    this.email,
    this.password,
  });

  @override
  Future<FormData?>? toFormData() {
    return null;
  }

  @override
  Map<String, dynamic>? toPostJson() {
    Map<String, dynamic> data = <String, dynamic>{};
    data["code"] = code;
    data["email"] = email;
    data["password"] = password;
    return data;
  }

  @override
  Map<String, dynamic>? toPutJson() {
    return null;
  }
}

class BlogBody extends BodyDataModel {
  String? storeId;
  String? title;
  String? content;
  List<String>? photos;
  List<String>? tags;

  BlogBody({
    this.title,
    this.content,
    this.storeId,
    this.photos,
    this.tags,
  });

  @override
  Map<String, dynamic>? toPostJson() {
    Map<String, dynamic> data = <String, dynamic>{};
    data["title"] = title;
    data["content"] = content;
    data["storeId"] = storeId;
    data["photos"] = photos;
    data["tags"] = tags;
    return data;
  }

  @override
  Map<String, dynamic>? toPutJson() {
    Map<String, dynamic> data = <String, dynamic>{};
    data["title"] = title;
    data["content"] = content;
    data["storeId"] = storeId;
    data["photos"] = photos;
    data["tags"] = tags;
    return data;
  }

  @override
  Future<FormData?>? toFormData() {
    return null;
  }
}

class BrandBody extends BodyDataModel {
  String? name;
  int? highlightRate;
  String? slug;
  String? description;
  String? photoId;
  List<String>? supplierIds;
  String? storeId;

  BrandBody({
    this.name,
    this.highlightRate,
    this.slug,
    this.description,
    this.photoId,
    this.supplierIds,
    this.storeId,
  });

  @override
  Map<String, dynamic>? toPostJson() {
    Map<String, dynamic> data = <String, dynamic>{};
    data["name"] = name;
    data["highlightRate"] = highlightRate;
    data["slug"] = slug;
    data["description"] = description;
    data["photoId"] = photoId;
    data["supplierIds"] = supplierIds;
    data["storeId"] = storeId;
    return data;
  }

  @override
  Map<String, dynamic>? toPutJson() {
    Map<String, dynamic> data = <String, dynamic>{};
    data["name"] = name;
    data["highlightRate"] = highlightRate;
    data["slug"] = slug;
    data["description"] = description;
    data["photoId"] = photoId;
    data["supplierIds"] = supplierIds;
    data["storeId"] = storeId;
    return data;
  }

  @override
  Future<FormData?>? toFormData() {
    return null;
  }
}

class CampaignBody extends BodyDataModel {
  String? title;
  int? highlightRate;
  String? slug;
  String? link;
  String? description;
  String? discountStartDate;
  String? discountEndDate;
  int? limit;
  bool? showOnMain;
  String? type;
  String? priceType;
  int? productCount;
  int? freeProductCount;
  String? photoId;
  List<String>? products;
  List<String>? brands;
  List<String>? categories;
  List<String>? tags;
  String? storeId;

  CampaignBody({
    this.title,
    this.highlightRate,
    this.slug,
    this.link,
    this.description,
    this.discountStartDate,
    this.discountEndDate,
    this.limit,
    this.showOnMain,
    this.type,
    this.priceType,
    this.productCount,
    this.freeProductCount,
    this.photoId,
    this.products,
    this.brands,
    this.categories,
    this.tags,
    this.storeId,
  });

  @override
  Map<String, dynamic>? toPostJson() {
    Map<String, dynamic> data = <String, dynamic>{};
    data["title"] = title;
    data["highlightRate"] = highlightRate;
    data["slug"] = slug;
    data["link"] = link;
    data["description"] = description;
    data["discountStartDate"] = discountStartDate;
    data["discountEndDate"] = discountEndDate;
    data["limit"] = limit;
    data["showOnMain"] = showOnMain;
    data["type"] = type;
    data["priceType"] = priceType;
    data["productCount"] = productCount;
    data["freeProductCount"] = freeProductCount;
    data["photoId"] = photoId;
    data["products"] = products;
    data["brands"] = brands;
    data["categories"] = categories;
    data["tags"] = tags;
    data["storeId"] = storeId;
    return data;
  }

  @override
  Map<String, dynamic>? toPutJson() {
    Map<String, dynamic> data = <String, dynamic>{};
    data["title"] = title;
    data["highlightRate"] = highlightRate;
    data["slug"] = slug;
    data["link"] = link;
    data["description"] = description;
    data["discountStartDate"] = discountStartDate;
    data["discountEndDate"] = discountEndDate;
    data["limit"] = limit;
    data["showOnMain"] = showOnMain;
    data["type"] = type;
    data["priceType"] = priceType;
    data["productCount"] = productCount;
    data["freeProductCount"] = freeProductCount;
    data["photoId"] = photoId;
    data["products"] = products;
    data["brands"] = brands;
    data["categories"] = categories;
    data["tags"] = tags;
    data["storeId"] = storeId;
    return data;
  }

  @override
  Future<FormData?>? toFormData() {
    return null;
  }
}

class ProductCatalogBody extends BodyDataModel {
  String? catalogId;
  String? productId;
  String? storeId;
  bool? canPublish;

  ProductCatalogBody({
    this.catalogId,
    this.productId,
    this.storeId,
    this.canPublish,
  });

  @override
  Map<String, dynamic>? toPostJson() {
    Map<String, dynamic> data = <String, dynamic>{};
    data["catalogId"] = catalogId;
    data["productId"] = productId;
    data["storeId"] = storeId;
    data["canPublish"] = canPublish;
    return data;
  }

  @override
  Map<String, dynamic>? toPutJson() {
    Map<String, dynamic> data = <String, dynamic>{};
    data["catalogId"] = catalogId;
    data["productId"] = productId;
    data["storeId"] = storeId;
    data["canPublish"] = canPublish;
    return data;
  }

  @override
  Future<FormData?>? toFormData() {
    return null;
  }
}

class AssignedCatalogBody extends BodyDataModel {
  String? storeId;
  String? assignedStoreId;
  String? catalogId;

  AssignedCatalogBody({
    this.storeId,
    this.assignedStoreId,
    this.catalogId,
  });

  @override
  Map<String, dynamic>? toPostJson() {
    Map<String, dynamic> data = <String, dynamic>{};
    data["storeId"] = storeId;
    data["assignedStoreId"] = assignedStoreId;
    data["catalogId"] = catalogId;
    return data;
  }

  @override
  Map<String, dynamic>? toPutJson() {
    Map<String, dynamic> data = <String, dynamic>{};
    data["storeId"] = storeId;
    data["assignedStoreId"] = assignedStoreId;
    data["catalogId"] = catalogId;
    return data;
  }

  @override
  Future<FormData?>? toFormData() {
    return null;
  }
}

class CatalogBody extends BodyDataModel {
  String? name;
  String? storeId;
  List<String>? userGroups;
  List<ProductCatalogBody>? products;
  List<String>? assignedStores;

  CatalogBody({
    this.name,
    this.storeId,
    this.userGroups,
    this.products,
    this.assignedStores,
  });

  @override
  Map<String, dynamic>? toPostJson() {
    Map<String, dynamic> data = <String, dynamic>{};
    data["name"] = name;
    data["storeId"] = storeId;
    data["userGroups"] = userGroups;
    data["products"] = products;
    data["assignedStores"] = assignedStores;
    return data;
  }

  @override
  Map<String, dynamic>? toPutJson() {
    Map<String, dynamic> data = <String, dynamic>{};
    data["name"] = name;
    data["storeId"] = storeId;
    data["userGroups"] = userGroups;
    data["products"] = products;
    data["assignedStores"] = assignedStores;
    return data;
  }

  @override
  Future<FormData?>? toFormData() {
    return null;
  }
}

class CategoryBody extends BodyDataModel {
  String? name;
  String? slug;
  int? sortIndex;
  String? parentId;
  String? taxId;
  String? storeId;
  List<String>? photos;

  CategoryBody({
    this.name,
    this.slug,
    this.sortIndex,
    this.parentId,
    this.taxId,
    this.storeId,
    this.photos,
  });

  @override
  Map<String, dynamic>? toPostJson() {
    Map<String, dynamic> data = <String, dynamic>{};
    data["name"] = name;
    data["slug"] = slug;
    data["sortIndex"] = sortIndex;
    data["parentId"] = parentId;
    data["taxId"] = taxId;
    data["storeId"] = storeId;
    data["photos"] = photos;
    return data;
  }

  @override
  Map<String, dynamic>? toPutJson() {
    Map<String, dynamic> data = <String, dynamic>{};
    data["name"] = name;
    data["slug"] = slug;
    data["sortIndex"] = sortIndex;
    data["parentId"] = parentId;
    data["taxId"] = taxId;
    data["storeId"] = storeId;
    data["photos"] = photos;
    return data;
  }

  @override
  Future<FormData?>? toFormData() {
    return null;
  }
}

class FeatureBody extends BodyDataModel {
  String? name;
  String? slug;
  String? description;
  String? icon;
  String? parentId;
  String? storeId;

  FeatureBody({
    this.name,
    this.slug,
    this.description,
    this.icon,
    this.parentId,
    this.storeId,
  });

  @override
  Map<String, dynamic>? toPostJson() {
    Map<String, dynamic> data = <String, dynamic>{};
    data["name"] = name;
    data["slug"] = slug;
    data["description"] = description;
    data["icon"] = icon;
    data["parentId"] = parentId;
    data["storeId"] = storeId;
    return data;
  }

  @override
  Map<String, dynamic>? toPutJson() {
    Map<String, dynamic> data = <String, dynamic>{};
    data["name"] = name;
    data["slug"] = slug;
    data["description"] = description;
    data["icon"] = icon;
    data["parentId"] = parentId;
    data["storeId"] = storeId;
    return data;
  }

  @override
  Future<FormData?>? toFormData() {
    return null;
  }
}

class MailTemplateBody extends BodyDataModel {
  String? storeId;
  String? name;
  String? title;
  String? content;
  String? design;

  MailTemplateBody({
    this.storeId,
    this.name,
    this.title,
    this.content,
    this.design,
  });

  @override
  Map<String, dynamic>? toPostJson() {
    Map<String, dynamic> data = <String, dynamic>{};
    data["storeId"] = storeId;
    data["name"] = name;
    data["title"] = title;
    data["content"] = content;
    data["design"] = design;
    return data;
  }

  @override
  Map<String, dynamic>? toPutJson() {
    Map<String, dynamic> data = <String, dynamic>{};
    data["storeId"] = storeId;
    data["name"] = name;
    data["title"] = title;
    data["content"] = content;
    data["design"] = design;
    return data;
  }

  @override
  Future<FormData?>? toFormData() {
    return null;
  }
}

class OrderRefundBody extends BodyDataModel {
  List<OrderRefundItemBody>? items;
  List<OrderRefundItemBody>? sideItems;

  OrderRefundBody({
    this.items,
    this.sideItems,
  });

  @override
  Map<String, dynamic>? toPostJson() {
    Map<String, dynamic> data = <String, dynamic>{};
    if (items != null) {
      data["items"] = items!.map((e) => e.toPostJson()).toList();
    }
    if (sideItems != null) {
      data["sideItems"] = sideItems!.map((e) => e.toPostJson()).toList();
    }
    return data;
  }

  @override
  Map<String, dynamic>? toPutJson() {
    return null;
  }

  @override
  Future<FormData?>? toFormData() {
    return null;
  }
}

class OrderRefundItemBody extends BodyDataModel {
  String? id;
  int? quantity;
  bool? includeToStock;
  String? note;

  OrderRefundItemBody({
    this.id,
    this.quantity,
    this.includeToStock,
    this.note,
  });

  @override
  Map<String, dynamic>? toPostJson() {
    Map<String, dynamic> data = <String, dynamic>{};
    data["id"] = id;
    data["quantity"] = quantity;
    data["includeToStock"] = includeToStock;
    data["note"] = note;
    return data;
  }

  @override
  Map<String, dynamic>? toPutJson() {
    return null;
  }

  @override
  Future<FormData?>? toFormData() {
    return null;
  }
}

class OrderPutBody extends BodyDataModel {
  String? id;
  String? storeId;
  String? status;
  String? type;
  String? note;
  AddressBody? billingAddress;
  AddressBody? shippingAddress;

  OrderPutBody({
    this.id,
    this.storeId,
    this.status,
    this.type,
    this.note,
    this.billingAddress,
    this.shippingAddress,
  });

  @override
  Map<String, dynamic>? toPostJson() {
    return null;
  }

  @override
  Map<String, dynamic>? toPutJson() {
    Map<String, dynamic> data = <String, dynamic>{};
    data["id"] = id;
    data["storeId"] = storeId;
    data["status"] = status;
    data["type"] = type;
    data["note"] = note;
    data["billingAddress"] = billingAddress?.toPostJson();
    data["shippingAddress"] = shippingAddress?.toPostJson();
    return data;
  }

  @override
  Future<FormData?>? toFormData() {
    return null;
  }
}

class AddressBody extends BodyDataModel {
  String? country;
  String? state;
  String? city;
  String? district;
  String? neighborhood;
  String? street;
  String? buildingNo;
  String? doorNo;
  String? zipCode;
  double? latitude;
  double? longitude;
  String? openAddress;
  String? id;

  AddressBody({
    this.country,
    this.state,
    this.city,
    this.district,
    this.neighborhood,
    this.street,
    this.buildingNo,
    this.doorNo,
    this.zipCode,
    this.latitude,
    this.longitude,
    this.openAddress,
    this.id,
  });

  @override
  Map<String, dynamic>? toPostJson() {
    Map<String, dynamic> data = <String, dynamic>{};
    data["country"] = country;
    data["state"] = state;
    data["city"] = city;
    data["district"] = district;
    data["neighborhood"] = neighborhood;
    data["street"] = street;
    data["buildingNo"] = buildingNo;
    data["doorNo"] = doorNo;
    data["zipCode"] = zipCode;
    data["latitude"] = latitude;
    data["longitude"] = longitude;
    data["openAddress"] = openAddress;
    data["id"] = id;
    return data;
  }

  @override
  Map<String, dynamic>? toPutJson() {
    return null;
  }

  @override
  Future<FormData?>? toFormData() {
    return null;
  }
}

class PhotoFolderBody extends BodyDataModel {
  String? name;
  String? parentId;
  String? storeId;

  PhotoFolderBody({
    this.name,
    this.parentId,
    this.storeId,
  });

  @override
  Map<String, dynamic>? toPostJson() {
    Map<String, dynamic> data = <String, dynamic>{};
    data['name'] = name;
    data['parentId'] = parentId;
    data['storeId'] = storeId;
    return data;
  }

  @override
  Map<String, dynamic>? toPutJson() {
    Map<String, dynamic> data = <String, dynamic>{};
    data['name'] = name;
    data['parentId'] = parentId;
    data['storeId'] = storeId;
    return data;
  }

  @override
  Future<FormData?>? toFormData() {
    return null;
  }
}

class PhotoBody extends BodyDataModel {
  String? name;
  String? description;
  String? alt;
  String? storeId;
  String? folderId;
  File? photo;

  PhotoBody({
    this.name,
    this.description,
    this.alt,
    this.storeId,
    this.folderId,
    this.photo,
  });

  @override
  Map<String, dynamic>? toPostJson() {
    return null;
  }

  @override
  Map<String, dynamic>? toPutJson() {
    return null;
  }

  @override
  Future<FormData?>? toFormData() async {
    FormData formData = FormData.fromMap({
      'name': name,
      'description': description,
      'alt': alt,
      'storeId': storeId,
      'folderId': folderId,
      'photo': await MultipartFile.fromFile(photo!.path,
          filename: photo!.path.split('/').last),
    });

    return formData;
  }
}

class PhotoMultipleBody extends BodyDataModel {
  String? storeId;
  String? folderId;
  List<File>? photos;

  PhotoMultipleBody({
    this.storeId,
    this.folderId,
    this.photos,
  });

  @override
  Map<String, dynamic>? toPostJson() {
    return null;
  }

  @override
  Map<String, dynamic>? toPutJson() {
    return null;
  }

  @override
  Future<FormData?>? toFormData() async {
    FormData formData = FormData.fromMap({
      'storeId': storeId,
      'folderId': folderId,
      'photos': await Future.wait(photos!
          .map((photo) async => await MultipartFile.fromFile(photo.path,
              filename: photo.path.split('/').last))
          .toList()),
    });

    return formData;
  }
}

class PriceGroupBody extends BodyDataModel {
  String? name;
  String? locale;
  String? storeId;

  PriceGroupBody({
    this.name,
    this.locale,
    this.storeId,
  });

  @override
  Map<String, dynamic>? toPostJson() {
    Map<String, dynamic> data = <String, dynamic>{};
    data['name'] = name;
    data['locale'] = locale;
    data['storeId'] = storeId;
    return data;
  }

  @override
  Map<String, dynamic>? toPutJson() {
    return null;
  }

  @override
  Future<FormData?>? toFormData() {
    return null;
  }
}

class PriceBody extends BodyDataModel {
  String? productId;
  double? price;
  double? discountedPrice;
  String? currency;
  String? taxId;
  String? groupId;
  String? storeId;

  PriceBody({
    this.productId,
    this.price,
    this.discountedPrice,
    this.currency,
    this.taxId,
    this.groupId,
    this.storeId,
  });

  @override
  Map<String, dynamic>? toPostJson() {
    Map<String, dynamic> data = <String, dynamic>{};
    data['productId'] = productId;
    data['price'] = price;
    data['discountedPrice'] = discountedPrice;
    data['currency'] = currency;
    data['taxId'] = taxId;
    data['groupId'] = groupId;
    data['storeId'] = storeId;
    return data;
  }

  @override
  Map<String, dynamic>? toPutJson() {
    return null;
  }

  @override
  Future<FormData?>? toFormData() {
    return null;
  }
}

class PriceMultiplePutBody extends BodyDataModel {
  String? productId;
  List<PriceBody>? prices;
  String? storeId;

  PriceMultiplePutBody({
    this.productId,
    this.prices,
    this.storeId,
  });

  @override
  Map<String, dynamic>? toPostJson() {
    return null;
  }

  @override
  Map<String, dynamic>? toPutJson() {
    Map<String, dynamic> data = <String, dynamic>{};
    data['productId'] = productId;
    if (prices != null) {
      data['prices'] = prices!.map((e) => e.toPostJson()).toList();
    }
    data['storeId'] = storeId;
    return data;
  }

  @override
  Future<FormData?>? toFormData() {
    return null;
  }
}

class ProductRatingPutBody extends BodyDataModel {
  bool? isVerified;
  List<String>? photos;

  ProductRatingPutBody({
    this.isVerified,
    this.photos,
  });

  @override
  Map<String, dynamic>? toPostJson() {
    return null;
  }

  @override
  Map<String, dynamic>? toPutJson() {
    Map<String, dynamic> data = <String, dynamic>{};
    data['isVerified'] = isVerified;
    data['photos'] = photos;
    return data;
  }

  @override
  Future<FormData?>? toFormData() {
    return null;
  }
}

class ProductBody extends BodyDataModel {
  String? storeId;
  String? groupId;
  bool? useStock;
  String? type;
  String? brandId;
  List<String>? categories;
  List<ProductDetailBody>? products;

  ProductBody({
    this.storeId,
    this.groupId,
    this.useStock,
    this.type,
    this.brandId,
    this.categories,
    this.products,
  });

  @override
  Map<String, dynamic>? toPostJson() {
    Map<String, dynamic> data = <String, dynamic>{};
    data['storeId'] = storeId;
    data['groupId'] = groupId;
    data['useStock'] = useStock;
    data['type'] = type;
    data['brandId'] = brandId;
    data['categories'] = categories;
    if (products != null) {
      data['products'] = products!.map((e) => e.toPostJson()).toList();
    }
    return data;
  }

  @override
  Map<String, dynamic>? toPutJson() {
    return null;
  }

  @override
  Future<FormData?>? toFormData() {
    return null;
  }
}

class ProductDetailBody extends BodyDataModel {
  ProductDetailBody({
    this.detail,
    this.isActive,
    this.showInList,
    this.showInHot,
    this.showInHome,
    this.sortIndex,
    this.modelCode,
    this.hsCode,
    this.variant1Id,
    this.variant2Id,
    this.sections,
    this.tags,
    this.features,
    this.photos,
    this.prices,
    this.stocks,
    this.sides,
    this.catalogs,
  });

  ProductDetailDetailBody? detail;
  bool? isActive;
  bool? showInList;
  bool? showInHot;
  bool? showInHome;
  int? sortIndex;
  String? modelCode;
  String? hsCode;
  String? variant1Id;
  String? variant2Id;
  List<ProductDetailSectionBody>? sections;
  List<String>? tags;
  List<String>? features;
  List<String>? photos;
  List<ProductDetailPriceBody>? prices;
  List<ProductDetailStockBody>? stocks;
  List<String>? sides;
  List<String>? catalogs;

  @override
  Map<String, dynamic>? toPostJson() {
    Map<String, dynamic> data = <String, dynamic>{};
    data['detail'] = detail?.toPostJson();
    data['isActive'] = isActive;
    data['showInList'] = showInList;
    data['showInHot'] = showInHot;
    data['showInHome'] = showInHome;
    data['sortIndex'] = sortIndex;
    data['modelCode'] = modelCode;
    data['hsCode'] = hsCode;
    data['variant1Id'] = variant1Id;
    data['variant2Id'] = variant2Id;
    data['sections'] = sections;
    data['tags'] = tags;
    data['features'] = features;
    data['photos'] = photos;
    data['prices'] = prices;
    data['stocks'] = stocks;
    data['sides'] = sides;
    data['catalogs'] = catalogs;
    return data;
  }

  @override
  Map<String, dynamic>? toPutJson() {
    return null;
  }

  @override
  Future<FormData?>? toFormData() {
    return null;
  }
}

class ProductDetailDetailBody extends BodyDataModel {
  ProductDetailDetailBody({
    this.name,
    this.description,
    this.barcode,
    this.slug,
  });

  String? name;
  String? description;
  String? barcode;
  String? slug;

  @override
  Map<String, dynamic>? toPostJson() {
    Map<String, dynamic> data = <String, dynamic>{};
    data['name'] = name;
    data['description'] = description;
    data['barcode'] = barcode;
    data['slug'] = slug;
    return data;
  }

  @override
  Map<String, dynamic>? toPutJson() {
    return null;
  }

  @override
  Future<FormData?>? toFormData() {
    return null;
  }
}

class ProductDetailSectionBody extends BodyDataModel {
  ProductDetailSectionBody({
    this.title,
    this.content,
  });

  String? title;
  String? content;

  @override
  Map<String, dynamic>? toPostJson() {
    Map<String, dynamic> data = <String, dynamic>{};
    data['title'] = title;
    data['content'] = content;
    return data;
  }

  @override
  Map<String, dynamic>? toPutJson() {
    return null;
  }

  @override
  Future<FormData?>? toFormData() {
    return null;
  }
}

class ProductDetailPriceBody extends BodyDataModel {
  ProductDetailPriceBody({
    this.price,
    this.discountedPrice,
    this.currency,
    this.taxId,
    this.groupId,
    this.storeId,
  });

  double? price;
  double? discountedPrice;
  String? currency;
  String? taxId;
  String? groupId;
  String? storeId;

  @override
  Map<String, dynamic>? toPostJson() {
    Map<String, dynamic> data = <String, dynamic>{};
    data['price'] = price;
    data['discountedPrice'] = discountedPrice;
    data['currency'] = currency;
    data['taxId'] = taxId;
    data['groupId'] = groupId;
    data['storeId'] = storeId;
    return data;
  }

  @override
  Map<String, dynamic>? toPutJson() {
    return null;
  }

  @override
  Future<FormData?>? toFormData() {
    return null;
  }
}

class ProductDetailStockBody extends BodyDataModel {
  ProductDetailStockBody({
    this.storeId,
    this.sku,
    this.quantity,
    this.note,
    this.startDate,
    this.endDate,
    this.purchaseDate,
    this.purchasePrice,
    this.purchaseCurrency,
    this.supplierId,
  });

  String? storeId;
  String? sku;
  int? quantity;
  String? note;
  String? startDate;
  String? endDate;
  String? purchaseDate;
  double? purchasePrice;
  String? purchaseCurrency;
  String? supplierId;

  @override
  Map<String, dynamic>? toPostJson() {
    Map<String, dynamic> data = <String, dynamic>{};
    data['storeId'] = storeId;
    data['sku'] = sku;
    data['quantity'] = quantity;
    data['note'] = note;
    data['startDate'] = startDate;
    data['endDate'] = endDate;
    data['purchaseDate'] = purchaseDate;
    data['purchasePrice'] = purchasePrice;
    data['purchaseCurrency'] = purchaseCurrency;
    data['supplierId'] = supplierId;
    return data;
  }

  @override
  Map<String, dynamic>? toPutJson() {
    return null;
  }

  @override
  Future<FormData?>? toFormData() {
    return null;
  }
}

class ProductMultiplePutBody extends BodyDataModel {
  String? storeId;
  List<ProductDetailBody>? products;

  ProductMultiplePutBody({
    this.storeId,
    this.products,
  });

  @override
  Map<String, dynamic>? toPostJson() {
    return null;
  }

  @override
  Map<String, dynamic>? toPutJson() {
    Map<String, dynamic> data = <String, dynamic>{};
    data['storeId'] = storeId;
    if (products != null) {
      data['products'] = products!.map((e) => e.toPostJson()).toList();
    }
    return data;
  }

  @override
  Future<FormData?>? toFormData() {
    return null;
  }
}

class ProfilePutBody extends BodyDataModel {
  String? name;
  String? surname;
  String? socialSecurityNo;

  ProfilePutBody({
    this.name,
    this.surname,
    this.socialSecurityNo,
  });

  @override
  Map<String, dynamic>? toPostJson() {
    return null;
  }

  @override
  Map<String, dynamic>? toPutJson() {
    Map<String, dynamic> data = <String, dynamic>{};
    data['name'] = name;
    data['surname'] = surname;
    data['socialSecurityNo'] = socialSecurityNo;
    return data;
  }

  @override
  Future<FormData?>? toFormData() {
    return null;
  }
}

class ProfilePasswordChangeBody extends BodyDataModel {
  String? oldPassword;
  String? newPassword;

  ProfilePasswordChangeBody({
    this.oldPassword,
    this.newPassword,
  });

  @override
  Map<String, dynamic>? toPostJson() {
    return null;
  }

  @override
  Map<String, dynamic>? toPutJson() {
    Map<String, dynamic> data = <String, dynamic>{};
    data['oldPassword'] = oldPassword;
    data['newPassword'] = newPassword;
    return data;
  }

  @override
  Future<FormData?>? toFormData() {
    return null;
  }
}

class ProfileEmailChangeBody extends BodyDataModel {
  String? email;
  String? password;

  ProfileEmailChangeBody({
    this.email,
    this.password,
  });

  @override
  Map<String, dynamic>? toPostJson() {
    return null;
  }

  @override
  Map<String, dynamic>? toPutJson() {
    Map<String, dynamic> data = <String, dynamic>{};
    data['email'] = email;
    data['password'] = password;
    return data;
  }

  @override
  Future<FormData?>? toFormData() {
    return null;
  }
}

class PromoCodeBody extends BodyDataModel {
  String? storeId;
  String? code;
  int? limit;
  int? amount;
  String? amountType;
  String? type;
  List<String>? products;
  List<String>? categories;

  PromoCodeBody({
    this.storeId,
    this.code,
    this.limit,
    this.amount,
    this.amountType,
    this.type,
    this.products,
    this.categories,
  });

  @override
  Map<String, dynamic>? toPostJson() {
    Map<String, dynamic> data = <String, dynamic>{};
    data['storeId'] = storeId;
    data['code'] = code;
    data['limit'] = limit;
    data['amount'] = amount;
    data['amountType'] = amountType;
    data['type'] = type;
    data['products'] = products;
    data['categories'] = categories;
    return data;
  }

  @override
  Map<String, dynamic>? toPutJson() {
    Map<String, dynamic> data = <String, dynamic>{};
    data['storeId'] = storeId;
    data['code'] = code;
    data['limit'] = limit;
    data['amount'] = amount;
    data['amountType'] = amountType;
    data['type'] = type;
    data['products'] = products;
    data['categories'] = categories;
    return data;
  }

  @override
  Future<FormData?>? toFormData() {
    return null;
  }
}

class RoleGroupBody extends BodyDataModel {
  String? name;
  String? storeId;

  RoleGroupBody({
    this.name,
    this.storeId,
  });

  @override
  Map<String, dynamic>? toPostJson() {
    Map<String, dynamic> data = <String, dynamic>{};
    data['name'] = name;
    data['storeId'] = storeId;
    return data;
  }

  @override
  Map<String, dynamic>? toPutJson() {
    Map<String, dynamic> data = <String, dynamic>{};
    data['name'] = name;
    data['storeId'] = storeId;
    return data;
  }

  @override
  Future<FormData?>? toFormData() {
    return null;
  }
}

class RoleBody extends BodyDataModel {
  String? name;
  List<RolePermissionBody>? permissions;
  String? groupId;
  String? storeId;

  RoleBody({
    this.name,
    this.permissions,
    this.groupId,
    this.storeId,
  });

  @override
  Map<String, dynamic>? toPostJson() {
    Map<String, dynamic> data = <String, dynamic>{};

    data['name'] = name;
    if (permissions != null) {
      data["permissions"] = permissions!.map((e) => e.toPostJson()).toList();
    }
    data['groupId'] = groupId;
    data['storeId'] = storeId;
    return data;
  }

  @override
  Map<String, dynamic>? toPutJson() {
    Map<String, dynamic> data = <String, dynamic>{};

    data['name'] = name;
    if (permissions != null) {
      data["permissions"] = permissions!.map((e) => e.toPostJson()).toList();
    }
    data['groupId'] = groupId;
    data['storeId'] = storeId;
    return data;
  }

  @override
  Future<FormData?>? toFormData() {
    return null;
  }
}

class RolePermissionBody extends BodyDataModel {
  String? path;
  List<String>? types;

  RolePermissionBody({
    this.path,
    this.types,
  });

  @override
  Map<String, dynamic>? toPostJson() {
    Map<String, dynamic> data = <String, dynamic>{};

    data['path'] = path;
    data['types'] = types;
    return data;
  }

  @override
  Map<String, dynamic>? toPutJson() {
    return null;
  }

  @override
  Future<FormData?>? toFormData() {
    return null;
  }
}

class InStockBody extends BodyDataModel {
  String? sku;
  String? productId;
  int? quantity;
  String? type;
  String? status;
  bool? include;
  String? note;
  String? startDate;
  String? endDate;
  String? purchaseDate;
  double? purchasePrice;
  String? purchaseCurrency;
  String? warehouseId;
  String? supplierId;
  String? storeId;

  InStockBody({
    this.sku,
    this.productId,
    this.quantity,
    this.type,
    this.status,
    this.include,
    this.note,
    this.startDate,
    this.endDate,
    this.purchaseDate,
    this.purchasePrice,
    this.purchaseCurrency,
    this.warehouseId,
    this.supplierId,
    this.storeId,
  });

  @override
  Map<String, dynamic>? toPostJson() {
    Map<String, dynamic> data = <String, dynamic>{};
    data['sku'] = sku;
    data['productId'] = productId;
    data['quantity'] = quantity;
    data['type'] = type;
    data['status'] = status;
    data['include'] = include;
    data['note'] = note;
    data['startDate'] = startDate;
    data['endDate'] = endDate;
    data['purchaseDate'] = purchaseDate;
    data['purchasePrice'] = purchasePrice;
    data['purchaseCurrency'] = purchaseCurrency;
    data['warehouseId'] = warehouseId;
    data['supplierId'] = supplierId;
    data['storeId'] = storeId;
    return data;
  }

  @override
  Map<String, dynamic>? toPutJson() {
    Map<String, dynamic> data = <String, dynamic>{};
    data['sku'] = sku;
    data['productId'] = productId;
    data['quantity'] = quantity;
    data['type'] = type;
    data['status'] = status;
    data['include'] = include;
    data['note'] = note;
    data['startDate'] = startDate;
    data['endDate'] = endDate;
    data['purchaseDate'] = purchaseDate;
    data['purchasePrice'] = purchasePrice;
    data['purchaseCurrency'] = purchaseCurrency;
    data['warehouseId'] = warehouseId;
    data['supplierId'] = supplierId;
    data['storeId'] = storeId;
    return data;
  }

  @override
  Future<FormData?>? toFormData() {
    return null;
  }
}

class OutStockBody extends BodyDataModel {
  String? sku;
  String? productId;
  int? quantity;
  String? type;
  String? status;
  bool? include;
  String? note;
  String? startDate;
  String? endDate;
  String? transactionDate;
  String? inStockId;
  String? warehouseId;
  String? supplierId;
  String? storeId;

  OutStockBody({
    this.sku,
    this.productId,
    this.quantity,
    this.type,
    this.status,
    this.include,
    this.note,
    this.startDate,
    this.endDate,
    this.transactionDate,
    this.inStockId,
    this.warehouseId,
    this.supplierId,
    this.storeId,
  });

  @override
  Map<String, dynamic>? toPostJson() {
    Map<String, dynamic> data = <String, dynamic>{};
    data['sku'] = sku;
    data['productId'] = productId;
    data['quantity'] = quantity;
    data['type'] = type;
    data['status'] = status;
    data['include'] = include;
    data['note'] = note;
    data['startDate'] = startDate;
    data['endDate'] = endDate;
    data['transactionDate'] = transactionDate;
    data['inStockId'] = inStockId;
    data['warehouseId'] = warehouseId;
    data['supplierId'] = supplierId;
    data['storeId'] = storeId;
    return data;
  }

  @override
  Map<String, dynamic>? toPutJson() {
    Map<String, dynamic> data = <String, dynamic>{};
    data['sku'] = sku;
    data['productId'] = productId;
    data['quantity'] = quantity;
    data['type'] = type;
    data['status'] = status;
    data['include'] = include;
    data['note'] = note;
    data['startDate'] = startDate;
    data['endDate'] = endDate;
    data['transactionDate'] = transactionDate;
    data['inStockId'] = inStockId;
    data['warehouseId'] = warehouseId;
    data['supplierId'] = supplierId;
    data['storeId'] = storeId;
    return data;
  }

  @override
  Future<FormData?>? toFormData() {
    return null;
  }
}

class StockListPutBody extends BodyDataModel {
  String? productId;
  String? startDate;
  String? endDate;
  String? storeId;

  StockListPutBody({
    this.productId,
    this.startDate,
    this.endDate,
    this.storeId,
  });

  @override
  Map<String, dynamic>? toPostJson() {
    return null;
  }

  @override
  Map<String, dynamic>? toPutJson() {
    Map<String, dynamic> data = <String, dynamic>{};

    data['productId'] = productId;
    data['startDate'] = startDate;
    data['endDate'] = endDate;
    data['storeId'] = storeId;
    return data;
  }

  @override
  Future<FormData?>? toFormData() {
    return null;
  }
}

class StoreRatingPutBody extends BodyDataModel {
  bool? isVerified;
  List<String>? photos;

  StoreRatingPutBody({
    this.isVerified,
    this.photos,
  });

  @override
  Map<String, dynamic>? toPostJson() {
    return null;
  }

  @override
  Map<String, dynamic>? toPutJson() {
    Map<String, dynamic> data = <String, dynamic>{};

    data['isVerified'] = isVerified;
    data['photos'] = photos;
    return data;
  }

  @override
  Future<FormData?>? toFormData() {
    return null;
  }
}

class StoreSettingCurrencyPutBody extends BodyDataModel {
  String? id;
  String? storeId;
  String? baseCurrency;
  List<StoreSettingCurrencyRatePutBody>? rates;

  StoreSettingCurrencyPutBody({
    this.id,
    this.storeId,
    this.baseCurrency,
    this.rates,
  });

  @override
  Map<String, dynamic>? toPostJson() {
    return null;
  }

  @override
  Map<String, dynamic>? toPutJson() {
    final Map<String, dynamic> data = <String, dynamic>{};
    data['id'] = id;
    data['storeId'] = storeId;
    data['baseCurrency'] = baseCurrency;
    if (rates != null) {
      data['rates'] = rates!.map((e) => e.toPostJson()).toList();
    }
    return data;
  }

  @override
  Future<FormData?>? toFormData() {
    return null;
  }
}

class StoreSettingCurrencyRatePutBody extends BodyDataModel {
  String? id;
  bool? isFixed;
  String? currency;
  double? rate;

  StoreSettingCurrencyRatePutBody({
    this.id,
    this.isFixed,
    this.currency,
    this.rate,
  });

  @override
  Map<String, dynamic>? toPostJson() {
    final Map<String, dynamic> data = <String, dynamic>{};
    data['id'] = id;
    data['isFixed'] = isFixed;
    data['currency'] = currency;
    data['rate'] = rate;
    return data;
  }

  @override
  Map<String, dynamic>? toPutJson() {
    return null;
  }

  @override
  Future<FormData?>? toFormData() {
    return null;
  }
}

class StoreSettingPutBody extends BodyDataModel {
  String? id;
  String? storeId;
  StoreSettingEmailPutBody? email;
  StoreSettingSmsPutBody? sms;
  StoreSettingCustomerPutBody? customer;
  StoreSettingCampaignPutBody? campaign;
  StoreSettingOrderPutBody? order;
  StoreSettingProductPutBody? product;
  StoreSettingCurrencyPutBody? currency;
  StoreSettingAnalyticPutBody? analytic;

  StoreSettingPutBody({
    this.id,
    this.storeId,
    this.email,
    this.sms,
    this.customer,
    this.campaign,
    this.order,
    this.product,
    this.currency,
    this.analytic,
  });

  @override
  Map<String, dynamic>? toPostJson() {
    final Map<String, dynamic> data = <String, dynamic>{};
    data['id'] = id;
    data['storeId'] = storeId;
    data['email'] = email?.toPostJson();
    data['sms'] = sms?.toPostJson();
    data['customer'] = customer?.toPostJson();
    data['campaign'] = campaign?.toPostJson();
    data['order'] = order?.toPostJson();
    data['product'] = product?.toPostJson();
    data['currency'] = currency?.toPostJson();
    data['analytic'] = analytic?.toPostJson();
    return data;
  }

  @override
  Map<String, dynamic>? toPutJson() {
    final Map<String, dynamic> data = <String, dynamic>{};
    data['id'] = id;
    data['storeId'] = storeId;
    data['email'] = email?.toPostJson();
    data['sms'] = sms?.toPostJson();
    data['customer'] = customer?.toPostJson();
    data['campaign'] = campaign?.toPostJson();
    data['order'] = order?.toPostJson();
    data['product'] = product?.toPostJson();
    data['currency'] = currency?.toPostJson();
    data['analytic'] = analytic?.toPostJson();
    return data;
  }

  @override
  Future<FormData?>? toFormData() {
    return null;
  }
}

class StoreSettingEmailPutBody extends BodyDataModel {
  String? SMTPHost;
  int? SMTPPort;
  bool? SMTPSecure;
  bool? SMTPTLS;
  String? SMTPEmail;
  String? SMTPPassword;
  bool? newCampaign;
  String? newCampaignMailTemplateId;
  bool? newOrder;
  String? newOrderMailTemplateId;
  bool? changeOrderStatus;
  String? changeOrderStatusMailTemplateId;

  StoreSettingEmailPutBody({
    this.SMTPHost,
    this.SMTPPort,
    this.SMTPSecure,
    this.SMTPTLS,
    this.SMTPEmail,
    this.SMTPPassword,
    this.newCampaign,
    this.newCampaignMailTemplateId,
    this.newOrder,
    this.newOrderMailTemplateId,
    this.changeOrderStatus,
    this.changeOrderStatusMailTemplateId,
  });

  @override
  Map<String, dynamic>? toPostJson() {
    final Map<String, dynamic> data = <String, dynamic>{};
    data['SMTPHost'] = SMTPHost;
    data['SMTPPort'] = SMTPPort;
    data['SMTPSecure'] = SMTPSecure;
    data['SMTPTLS'] = SMTPTLS;
    data['SMTPEmail'] = SMTPEmail;
    data['SMTPPassword'] = SMTPPassword;
    data['newCampaign'] = newCampaign;
    data['newCampaignMailTemplateId'] = newCampaignMailTemplateId;
    data['newOrder'] = newOrder;
    data['newOrderMailTemplateId'] = newOrderMailTemplateId;
    data['changeOrderStatus'] = changeOrderStatus;
    data['changeOrderStatusMailTemplateId'] = changeOrderStatusMailTemplateId;
    return data;
  }

  @override
  Map<String, dynamic>? toPutJson() {
    final Map<String, dynamic> data = <String, dynamic>{};
    data['SMTPHost'] = SMTPHost;
    data['SMTPPort'] = SMTPPort;
    data['SMTPSecure'] = SMTPSecure;
    data['SMTPTLS'] = SMTPTLS;
    data['SMTPEmail'] = SMTPEmail;
    data['SMTPPassword'] = SMTPPassword;
    data['newCampaign'] = newCampaign;
    data['newCampaignMailTemplateId'] = newCampaignMailTemplateId;
    data['newOrder'] = newOrder;
    data['newOrderMailTemplateId'] = newOrderMailTemplateId;
    data['changeOrderStatus'] = changeOrderStatus;
    data['changeOrderStatusMailTemplateId'] = changeOrderStatusMailTemplateId;
    return data;
  }

  @override
  Future<FormData?>? toFormData() {
    return null;
  }
}

class StoreSettingSmsPutBody extends BodyDataModel {
  bool? newCampaign;
  bool? newOrder;
  bool? changeOrderStatus;

  StoreSettingSmsPutBody({
    this.newCampaign,
    this.newOrder,
    this.changeOrderStatus,
  });

  @override
  Map<String, dynamic>? toPostJson() {
    final Map<String, dynamic> data = <String, dynamic>{};
    data['newCampaign'] = newCampaign;
    data['newOrder'] = newOrder;
    data['changeOrderStatus'] = changeOrderStatus;
    return data;
  }

  @override
  Map<String, dynamic>? toPutJson() {
    final Map<String, dynamic> data = <String, dynamic>{};
    data['newCampaign'] = newCampaign;
    data['newOrder'] = newOrder;
    data['changeOrderStatus'] = changeOrderStatus;
    return data;
  }

  @override
  Future<FormData?>? toFormData() {
    return null;
  }
}

class StoreSettingCustomerPutBody extends BodyDataModel {
  bool? verifyUsers;

  StoreSettingCustomerPutBody({
    this.verifyUsers,
  });

  @override
  Map<String, dynamic>? toPostJson() {
    final Map<String, dynamic> data = <String, dynamic>{};
    data['verifyUsers'] = verifyUsers;
    return data;
  }

  @override
  Map<String, dynamic>? toPutJson() {
    final Map<String, dynamic> data = <String, dynamic>{};
    data['verifyUsers'] = verifyUsers;
    return data;
  }

  @override
  Future<FormData?>? toFormData() {
    return null;
  }
}

class StoreSettingCampaignPutBody extends BodyDataModel {
  bool? customersCanUseMultipleCampaign;

  StoreSettingCampaignPutBody({
    this.customersCanUseMultipleCampaign,
  });

  @override
  Map<String, dynamic>? toPostJson() {
    final Map<String, dynamic> data = <String, dynamic>{};
    data['customersCanUseMultipleCampaign'] = customersCanUseMultipleCampaign;
    return data;
  }

  @override
  Map<String, dynamic>? toPutJson() {
    final Map<String, dynamic> data = <String, dynamic>{};
    data['customersCanUseMultipleCampaign'] = customersCanUseMultipleCampaign;
    return data;
  }

  @override
  Future<FormData?>? toFormData() {
    return null;
  }
}

class StoreSettingOrderPutBody extends BodyDataModel {
  bool? guestOrder;
  bool? canCreateOrder;
  String? invoicePrefix;
  int? invoiceStartNumber;
  int? minimumCartAmount;

  StoreSettingOrderPutBody({
    this.guestOrder,
    this.canCreateOrder,
    this.invoicePrefix,
    this.invoiceStartNumber,
    this.minimumCartAmount,
  });

  @override
  Map<String, dynamic>? toPostJson() {
    final Map<String, dynamic> data = <String, dynamic>{};
    data['guestOrder'] = guestOrder;
    data['canCreateOrder'] = canCreateOrder;
    data['invoicePrefix'] = invoicePrefix;
    data['invoiceStartNumber'] = invoiceStartNumber;
    data['minimumCartAmount'] = minimumCartAmount;
    return data;
  }

  @override
  Map<String, dynamic>? toPutJson() {
    final Map<String, dynamic> data = <String, dynamic>{};
    data['guestOrder'] = guestOrder;
    data['canCreateOrder'] = canCreateOrder;
    data['invoicePrefix'] = invoicePrefix;
    data['invoiceStartNumber'] = invoiceStartNumber;
    data['minimumCartAmount'] = minimumCartAmount;
    return data;
  }

  @override
  Future<FormData?>? toFormData() {
    return null;
  }
}

class StoreSettingProductPutBody extends BodyDataModel {
  bool? verifyRatings;
  bool? showNoStocks;
  String? showPrice;

  StoreSettingProductPutBody({
    this.verifyRatings,
    this.showNoStocks,
    this.showPrice,
  });

  @override
  Map<String, dynamic>? toPostJson() {
    final Map<String, dynamic> data = <String, dynamic>{};
    data['verifyRatings'] = verifyRatings;
    data['showNoStocks'] = showNoStocks;
    data['showPrice'] = showPrice;
    return data;
  }

  @override
  Map<String, dynamic>? toPutJson() {
    final Map<String, dynamic> data = <String, dynamic>{};
    data['verifyRatings'] = verifyRatings;
    data['showNoStocks'] = showNoStocks;
    data['showPrice'] = showPrice;
    return data;
  }

  @override
  Future<FormData?>? toFormData() {
    return null;
  }
}

class StoreSettingAnalyticPutBody extends BodyDataModel {
  String? googleTagManagerId;
  String? googleAnalyticsId;
  String? facebookPixelId;

  StoreSettingAnalyticPutBody({
    this.googleTagManagerId,
    this.googleAnalyticsId,
    this.facebookPixelId,
  });

  @override
  Map<String, dynamic>? toPostJson() {
    final Map<String, dynamic> data = <String, dynamic>{};
    data['googleTagManagerId'] = googleTagManagerId;
    data['googleAnalyticsId'] = googleAnalyticsId;
    data['facebookPixelId'] = facebookPixelId;
    return data;
  }

  @override
  Map<String, dynamic>? toPutJson() {
    final Map<String, dynamic> data = <String, dynamic>{};
    data['googleTagManagerId'] = googleTagManagerId;
    data['googleAnalyticsId'] = googleAnalyticsId;
    data['facebookPixelId'] = facebookPixelId;
    return data;
  }

  @override
  Future<FormData?>? toFormData() {
    return null;
  }
}

class StoreShipmentMethodBody extends BodyDataModel {
  String? storeId;
  String? name;
  String? logoURL;
  String? type;
  String? calculationType;
  int? shipmentEstimatedDay;
  bool? showShipmentEstimatedDay;
  int? freeShipmentCargoLimit;
  StoreShipmentMethodCashOnDeliveryBody? cashOnDelivery;
  List<StoreShipmentMethodPriceBody>? prices;

  StoreShipmentMethodBody({
    this.storeId,
    this.name,
    this.logoURL,
    this.type,
    this.calculationType,
    this.shipmentEstimatedDay,
    this.showShipmentEstimatedDay,
    this.freeShipmentCargoLimit,
    this.cashOnDelivery,
    this.prices,
  });

  @override
  Map<String, dynamic>? toPostJson() {
    final Map<String, dynamic> data = <String, dynamic>{};

    data['storeId'] = storeId;
    data['name'] = name;
    data['logoURL'] = logoURL;
    data['type'] = type;
    data['calculationType'] = calculationType;
    data['shipmentEstimatedDay'] = shipmentEstimatedDay;
    data['showShipmentEstimatedDay'] = showShipmentEstimatedDay;
    data['freeShipmentCargoLimit'] = freeShipmentCargoLimit;
    data['cashOnDelivery'] = cashOnDelivery?.toPostJson();

    if (prices != null) {
      data["prices"] = prices!.map((e) => e.toPostJson()).toList();
    }
    return data;
  }

  @override
  Map<String, dynamic>? toPutJson() {
    final Map<String, dynamic> data = <String, dynamic>{};

    data['storeId'] = storeId;
    data['name'] = name;
    data['logoURL'] = logoURL;
    data['type'] = type;
    data['calculationType'] = calculationType;
    data['shipmentEstimatedDay'] = shipmentEstimatedDay;
    data['showShipmentEstimatedDay'] = showShipmentEstimatedDay;
    data['freeShipmentCargoLimit'] = freeShipmentCargoLimit;
    data['cashOnDelivery'] = cashOnDelivery?.toPostJson();
    if (prices != null) {
      data["prices"] = prices!.map((e) => e.toPostJson()).toList();
    }
    return data;
  }

  @override
  Future<FormData?>? toFormData() {
    return null;
  }
}

class StoreShipmentMethodCashOnDeliveryBody extends BodyDataModel {
  String? description;
  List<StoreShipmentMethodCashOnDeliveryPriceBody>? prices;

  StoreShipmentMethodCashOnDeliveryBody({
    this.description,
    this.prices,
  });

  @override
  Map<String, dynamic>? toPostJson() {
    final Map<String, dynamic> data = <String, dynamic>{};

    data['description'] = description;
    if (prices != null) {
      data["prices"] = prices!.map((e) => e.toPostJson()).toList();
    }
    return data;
  }

  @override
  Map<String, dynamic>? toPutJson() {
    return null;
  }

  @override
  Future<FormData?>? toFormData() {
    return null;
  }
}

class StoreShipmentMethodCashOnDeliveryPriceBody extends BodyDataModel {
  String? currency;
  int? startCartAmount;
  int? endCartAmount;
  double? price;

  StoreShipmentMethodCashOnDeliveryPriceBody({
    this.currency,
    this.startCartAmount,
    this.endCartAmount,
    this.price,
  });

  @override
  Map<String, dynamic>? toPostJson() {
    final Map<String, dynamic> data = <String, dynamic>{};

    data['currency'] = currency;
    data['startCartAmount'] = startCartAmount;
    data['endCartAmount'] = endCartAmount;
    data['price'] = price;
    return data;
  }

  @override
  Map<String, dynamic>? toPutJson() {
    return null;
  }

  @override
  Future<FormData?>? toFormData() {
    return null;
  }
}

class StoreShipmentMethodPriceBody extends BodyDataModel {
  String? description;
  int? startCartAmount;
  int? endCartAmount;
  double? price;
  double? taxPercentage;
  String? currency;
  String? country;
  String? state;
  String? city;
  String? district;

  StoreShipmentMethodPriceBody({
    this.description,
    this.startCartAmount,
    this.endCartAmount,
    this.price,
    this.taxPercentage,
    this.currency,
    this.country,
    this.state,
    this.city,
    this.district,
  });

  @override
  Map<String, dynamic>? toPostJson() {
    final Map<String, dynamic> data = <String, dynamic>{};

    data['description'] = description;
    data['startCartAmount'] = startCartAmount;
    data['endCartAmount'] = endCartAmount;
    data['price'] = price;
    data['taxPercentage'] = taxPercentage;
    data['currency'] = currency;
    data['country'] = country;
    data['state'] = state;
    data['city'] = city;
    data['district'] = district;
    return data;
  }

  @override
  Map<String, dynamic>? toPutJson() {
    return null;
  }

  @override
  Future<FormData?>? toFormData() {
    return null;
  }
}

class StoreTypeBody extends BodyDataModel {
  String? storeId;
  String? name;
  String? slug;

  StoreTypeBody({
    this.storeId,
    this.name,
    this.slug,
  });

  @override
  Map<String, dynamic>? toPostJson() {
    final Map<String, dynamic> data = <String, dynamic>{};

    data['storeId'] = storeId;
    data['name'] = name;
    data['slug'] = slug;
    return data;
  }

  @override
  Map<String, dynamic>? toPutJson() {
    return null;
  }

  @override
  Future<FormData?>? toFormData() {
    return null;
  }
}

class SupplierBody extends BodyDataModel {
  String? name;
  List<String>? brandIds;
  List<String>? productGroupIds;
  String? storeId;

  SupplierBody({
    this.name,
    this.brandIds,
    this.productGroupIds,
    this.storeId,
  });

  @override
  Map<String, dynamic>? toPostJson() {
    final Map<String, dynamic> data = <String, dynamic>{};

    data['name'] = name;
    data['brandIds'] = brandIds;
    data['productGroupIds'] = productGroupIds;
    data['storeId'] = storeId;
    return data;
  }

  @override
  Map<String, dynamic>? toPutJson() {
    final Map<String, dynamic> data = <String, dynamic>{};

    data['name'] = name;
    data['brandIds'] = brandIds;
    data['productGroupIds'] = productGroupIds;
    data['storeId'] = storeId;
    return data;
  }

  @override
  Future<FormData?>? toFormData() {
    return null;
  }
}

class TagBody extends BodyDataModel {
  String? name;
  String? slug;
  String? description;
  String? type;
  String? parentId;
  String? photoId;
  String? storeId;

  TagBody({
    this.name,
    this.slug,
    this.description,
    this.type,
    this.parentId,
    this.photoId,
    this.storeId,
  });

  @override
  Map<String, dynamic>? toPostJson() {
    final Map<String, dynamic> data = <String, dynamic>{};

    data['name'] = name;
    data['slug'] = slug;
    data['description'] = description;
    data['type'] = type;
    data['parentId'] = parentId;
    data['photoId'] = photoId;
    data['storeId'] = storeId;
    return data;
  }

  @override
  Map<String, dynamic>? toPutJson() {
    final Map<String, dynamic> data = <String, dynamic>{};

    data['name'] = name;
    data['slug'] = slug;
    data['description'] = description;
    data['type'] = type;
    data['parentId'] = parentId;
    data['photoId'] = photoId;
    data['storeId'] = storeId;
    return data;
  }

  @override
  Future<FormData?>? toFormData() {
    return null;
  }
}

class TaxBody extends BodyDataModel {
  String? name;
  double? percentage;
  String? storeId;

  TaxBody({
    this.name,
    this.percentage,
    this.storeId,
  });

  @override
  Map<String, dynamic>? toPostJson() {
    final Map<String, dynamic> data = <String, dynamic>{};

    data["name"] = name;
    data["percentage"] = percentage;
    data["storeId"] = storeId;
    return data;
  }

  @override
  Map<String, dynamic>? toPutJson() {
    return null;
  }

  @override
  Future<FormData?>? toFormData() {
    return null;
  }
}

class ThemeBody extends BodyDataModel {
  String? storeId;
  String? name;
  String? title;
  String? content;
  String? design;

  ThemeBody({
    this.storeId,
    this.name,
    this.title,
    this.content,
    this.design,
  });

  @override
  Map<String, dynamic>? toPostJson() {
    final Map<String, dynamic> data = <String, dynamic>{};
    data["storeId"] = storeId;
    data["name"] = name;
    data["title"] = title;
    data["content"] = content;
    data["design"] = design;
    return data;
  }

  @override
  Map<String, dynamic>? toPutJson() {
    return null;
  }

  @override
  Future<FormData?>? toFormData() {
    return null;
  }
}

class TicketCategoryBody extends BodyDataModel {
  String? name;

  TicketCategoryBody({
    this.name,
  });

  @override
  Map<String, dynamic>? toPostJson() {
    final Map<String, dynamic> data = <String, dynamic>{};
    data["name"] = name;
    return data;
  }

  @override
  Map<String, dynamic>? toPutJson() {
    return null;
  }

  @override
  Future<FormData?>? toFormData() {
    return null;
  }
}

class TicketBody extends BodyDataModel {
  String? storeId;
  String? title;
  String? message;
  String? categoryId;
  List<File>? files;

  TicketBody({
    this.storeId,
    this.title,
    this.message,
    this.categoryId,
    this.files,
  });

  @override
  Map<String, dynamic>? toPostJson() {
    return null;
  }

  @override
  Map<String, dynamic>? toPutJson() {
    return null;
  }

  @override
  Future<FormData?>? toFormData() async {
    FormData formData = FormData.fromMap({
      'storeId': storeId,
      'title': title,
      'message': message,
      'categoryId': categoryId,
      'files': files != null
          ? await Future.wait(files!
              .map((file) async => MultipartFile.fromBytes(
                  file.readAsBytesSync(),
                  filename: file.path.split('/').last))
              .toList())
          : null,
    });

    return formData;
  }
}

class TicketMessageBody extends BodyDataModel {
  String? storeId;
  String? message;
  String? status;
  List<File>? files;

  TicketMessageBody({
    this.storeId,
    this.message,
    this.status,
    this.files,
  });

  @override
  Map<String, dynamic>? toPostJson() {
    return null;
  }

  @override
  Map<String, dynamic>? toPutJson() {
    return null;
  }

  @override
  Future<FormData?>? toFormData() async {
    FormData formData = FormData.fromMap({
      'storeId': storeId,
      'message': message,
      'status': status,
      'files': files != null
          ? await Future.wait(files!
              .map((file) async => MultipartFile.fromBytes(
                  file.readAsBytesSync(),
                  filename: file.path.split('/').last))
              .toList())
          : null,
    });

    return formData;
  }
}

class TicketAssignPutBody extends BodyDataModel {
  List<String>? userIds;

  TicketAssignPutBody({
    this.userIds,
  });

  @override
  Map<String, dynamic>? toPostJson() {
    return null;
  }

  @override
  Map<String, dynamic>? toPutJson() {
    Map<String, dynamic> data = <String, dynamic>{};
    if (userIds != null) {
      data["userIds"] = userIds!.map((e) => e).toList();
    }

    return data;
  }

  @override
  Future<FormData?>? toFormData() {
    return null;
  }
}

class TransferBody extends BodyDataModel {
  String? note;
  String? startWarehouseId;
  String? destinationWarehouseId;
  String? vehicleId;
  String? storeId;
  List<TransferStockBody>? stocks;

  TransferBody({
    this.note,
    this.startWarehouseId,
    this.destinationWarehouseId,
    this.vehicleId,
    this.storeId,
    this.stocks,
  });

  @override
  Map<String, dynamic>? toPostJson() {
    Map<String, dynamic> data = <String, dynamic>{};
    data["note"] = note;
    data["startWarehouseId"] = startWarehouseId;
    data["destinationWarehouseId"] = destinationWarehouseId;
    data["vehicleId"] = vehicleId;
    data["storeId"] = storeId;
    if (stocks != null) {
      data["stocks"] = stocks!.map((item) => item.toPostJson()!).toList();
    }
    return data;
  }

  @override
  Map<String, dynamic>? toPutJson() {
    Map<String, dynamic> data = <String, dynamic>{};
    data["note"] = note;
    data["startWarehouseId"] = startWarehouseId;
    data["destinationWarehouseId"] = destinationWarehouseId;
    data["vehicleId"] = vehicleId;
    data["storeId"] = storeId;
    if (stocks != null) {
      data["stocks"] = stocks!.map((item) => item.toPostJson()!).toList();
    }
    return data;
  }

  @override
  Future<FormData?>? toFormData() {
    return null;
  }
}

class TransferStockBody extends BodyDataModel {
  String? id;
  int? quantity;
  String? note;

  TransferStockBody({
    this.id,
    this.quantity,
    this.note,
  });

  @override
  Map<String, dynamic>? toPostJson() {
    Map<String, dynamic> data = <String, dynamic>{};
    data["id"] = id;
    data["quantity"] = quantity;
    data["note"] = note;
    return data;
  }

  @override
  Map<String, dynamic>? toPutJson() {
    Map<String, dynamic> data = <String, dynamic>{};
    data["id"] = id;
    data["quantity"] = quantity;
    data["note"] = note;
    return data;
  }

  @override
  Future<FormData?>? toFormData() {
    return null;
  }
}

class TransferCompleteBody extends BodyDataModel {
  List<TransferStockBody>? stocks;

  TransferCompleteBody({
    this.stocks,
  });

  @override
  Map<String, dynamic>? toPostJson() {
    Map<String, dynamic> data = <String, dynamic>{};
    if (stocks != null) {
      data["stocks"] = stocks!.map((item) => item.toPostJson()!).toList();
    }
    return data;
  }

  @override
  Map<String, dynamic>? toPutJson() {
    Map<String, dynamic> data = <String, dynamic>{};
    if (stocks != null) {
      data["stocks"] = stocks!.map((item) => item.toPostJson()!).toList();
    }
    return data;
  }

  @override
  Future<FormData?>? toFormData() {
    return null;
  }
}

class UserGroupBody extends BodyDataModel {
  String? storeId;
  String? name;
  double? baseDiscountPercentage;
  String? priceGroupId;
  List<String>? users;

  UserGroupBody({
    this.storeId,
    this.name,
    this.baseDiscountPercentage,
    this.priceGroupId,
    this.users,
  });

  @override
  Map<String, dynamic>? toPostJson() {
    Map<String, dynamic> data = <String, dynamic>{};
    data["storeId"] = storeId;
    data["name"] = name;
    data["baseDiscountPercentage"] = baseDiscountPercentage;
    data["priceGroupId"] = priceGroupId;
    data["users"] = users;
    return data;
  }

  @override
  Map<String, dynamic>? toPutJson() {
    Map<String, dynamic> data = <String, dynamic>{};
    data["storeId"] = storeId;
    data["name"] = name;
    data["baseDiscountPercentage"] = baseDiscountPercentage;
    data["priceGroupId"] = priceGroupId;
    data["users"] = users;
    return data;
  }

  @override
  Future<FormData?>? toFormData() {
    return null;
  }
}

class UserBody extends BodyDataModel {
  String? email;
  String? password;
  String? type;
  UserDetailBody? detail;
  String? storeId;
  List<String>? stores;
  List<UserRoleBody>? roles;

  UserBody({
    this.email,
    this.password,
    this.type,
    this.detail,
    this.storeId,
    this.stores,
    this.roles,
  });

  @override
  Map<String, dynamic>? toPostJson() {
    Map<String, dynamic> data = <String, dynamic>{};
    data["email"] = email;
    data["password"] = password;
    data["type"] = type;
    data["detail"] = detail?.toPostJson();
    data["storeId"] = storeId;
    data["stores"] = stores;
    if (roles != null) {
      data["roles"] = roles!.map((item) => item.toPostJson()!).toList();
    }
    return data;
  }

  @override
  Map<String, dynamic>? toPutJson() {
    Map<String, dynamic> data = <String, dynamic>{};
    data["email"] = email;
    data["password"] = password;
    data["type"] = type;
    data["detail"] = detail?.toPostJson();
    data["storeId"] = storeId;
    data["stores"] = stores;
    if (roles != null) {
      data["roles"] = roles!.map((item) => item.toPostJson()!).toList();
    }
    return data;
  }

  @override
  Future<FormData?>? toFormData() {
    return null;
  }
}

class UserDetailBody extends BodyDataModel {
  String? name;
  String? surname;

  UserDetailBody({
    this.name,
    this.surname,
  });

  @override
  Map<String, dynamic>? toPostJson() {
    Map<String, dynamic> data = <String, dynamic>{};
    data["name"] = name;
    data["surname"] = surname;
    return data;
  }

  @override
  Map<String, dynamic>? toPutJson() {
    Map<String, dynamic> data = <String, dynamic>{};
    data["name"] = name;
    data["surname"] = surname;
    return data;
  }

  @override
  Future<FormData?>? toFormData() {
    return null;
  }
}

class UserRoleBody extends BodyDataModel {
  String? id;
  String? storeId;

  UserRoleBody({
    this.id,
    this.storeId,
  });

  @override
  Map<String, dynamic>? toPostJson() {
    Map<String, dynamic> data = <String, dynamic>{};
    data["id"] = id;
    data["storeId"] = storeId;
    return data;
  }

  @override
  Map<String, dynamic>? toPutJson() {
    return null;
  }

  @override
  Future<FormData?>? toFormData() {
    return null;
  }
}

class VariantGroupBody extends BodyDataModel {
  String? name;
  bool? isGrouped;
  String? storeId;

  VariantGroupBody({
    this.name,
    this.isGrouped,
    this.storeId,
  });

  @override
  Map<String, dynamic>? toPostJson() {
    Map<String, dynamic> data = <String, dynamic>{};

    data["name"] = name;
    data["isGrouped"] = isGrouped;
    data["storeId"] = storeId;
    return data;
  }

  @override
  Map<String, dynamic>? toPutJson() {
    Map<String, dynamic> data = <String, dynamic>{};

    data["name"] = name;
    data["isGrouped"] = isGrouped;
    data["storeId"] = storeId;
    return data;
  }

  @override
  Future<FormData?>? toFormData() {
    return null;
  }
}

class VariantBody extends BodyDataModel {
  String? name;
  String? groupId;
  String? storeId;

  VariantBody({
    this.name,
    this.groupId,
    this.storeId,
  });

  @override
  Map<String, dynamic>? toPostJson() {
    Map<String, dynamic> data = <String, dynamic>{};

    data["name"] = name;
    data["groupId"] = groupId;
    data["storeId"] = storeId;
    return data;
  }

  @override
  Map<String, dynamic>? toPutJson() {
    Map<String, dynamic> data = <String, dynamic>{};

    data["name"] = name;
    data["groupId"] = groupId;
    data["storeId"] = storeId;
    return data;
  }

  @override
  Future<FormData?>? toFormData() {
    return null;
  }
}

class VariantMultipleBody extends BodyDataModel {
  List<String>? names;
  String? groupId;
  String? storeId;

  VariantMultipleBody({
    this.names,
    this.groupId,
    this.storeId,
  });

  @override
  Map<String, dynamic>? toPostJson() {
    Map<String, dynamic> data = <String, dynamic>{};
    if (names != null) {
      data["names"] = names!.map((e) => e).toList();
    }
    data["groupId"] = groupId;
    data["storeId"] = storeId;
    return data;
  }

  @override
  Map<String, dynamic>? toPutJson() {
    Map<String, dynamic> data = <String, dynamic>{};

    if (names != null) {
      data["names"] = names!.map((e) => e).toList();
    }
    data["groupId"] = groupId;
    data["storeId"] = storeId;
    return data;
  }

  @override
  Future<FormData?>? toFormData() {
    return null;
  }
}

class VehicleBody extends BodyDataModel {
  String? licensePlate;
  String? brand;
  String? model;
  String? year;
  String? chauffeurName;
  String? chauffeurSurname;
  String? chauffeurPhoneNumber;
  String? storeId;

  VehicleBody({
    this.licensePlate,
    this.brand,
    this.model,
    this.year,
    this.chauffeurName,
    this.chauffeurSurname,
    this.chauffeurPhoneNumber,
    this.storeId,
  });

  @override
  Map<String, dynamic>? toPostJson() {
    Map<String, dynamic> data = <String, dynamic>{};

    data["licensePlate"] = licensePlate;
    data["brand"] = brand;
    data["model"] = model;
    data["year"] = year;
    data["chauffeurName"] = chauffeurName;
    data["chauffeurSurname"] = chauffeurSurname;
    data["chauffeurPhoneNumber"] = chauffeurPhoneNumber;
    data["storeId"] = storeId;
    return data;
  }

  @override
  Map<String, dynamic>? toPutJson() {
    Map<String, dynamic> data = <String, dynamic>{};

    data["licensePlate"] = licensePlate;
    data["brand"] = brand;
    data["model"] = model;
    data["year"] = year;
    data["chauffeurName"] = chauffeurName;
    data["chauffeurSurname"] = chauffeurSurname;
    data["chauffeurPhoneNumber"] = chauffeurPhoneNumber;
    data["storeId"] = storeId;
    return data;
  }

  @override
  Future<FormData?>? toFormData() {
    return null;
  }
}

class WarehouseBody extends BodyDataModel {
  String? name;
  AddressBody? address;
  String? storeId;

  WarehouseBody({
    this.name,
    this.address,
    this.storeId,
  });

  @override
  Map<String, dynamic>? toPostJson() {
    Map<String, dynamic> data = <String, dynamic>{};

    data["name"] = name;
    data["address"] = address?.toPostJson();
    data["storeId"] = storeId;
    return data;
  }

  @override
  Map<String, dynamic>? toPutJson() {
    Map<String, dynamic> data = <String, dynamic>{};

    data["name"] = name;
    data["address"] = address?.toPostJson();
    data["storeId"] = storeId;
    return data;
  }

  @override
  Future<FormData?>? toFormData() {
    return null;
  }
}

class ProductCatalogMultipleBody extends BodyDataModel {
  String? catalogId;
  String? storeId;
  bool? canPublish;
  List<String>? products;

  ProductCatalogMultipleBody({
    this.catalogId,
    this.storeId,
    this.canPublish,
    this.products,
  });

  @override
  Map<String, dynamic>? toPostJson() {
    Map<String, dynamic> data = <String, dynamic>{};
    data["catalogId"] = catalogId;
    data["storeId"] = storeId;
    data["canPublish"] = canPublish;
    if (products != null) {
      data["products"] = products!.map((e) => e).toList();
    }
    return data;
  }

  @override
  Map<String, dynamic>? toPutJson() {
    return null;
  }

  @override
  Future<FormData?>? toFormData() {
    return null;
  }
}

class ProductMultipleDeleteBody extends BodyDataModel {
  List<String>? ids;

  ProductMultipleDeleteBody({
    this.ids,
  });

  @override
  Map<String, dynamic>? toPostJson() {
    Map<String, dynamic> data = <String, dynamic>{};

    if (ids != null) {
      data["ids"] = ids!.map((e) => e).toList();
    }
    return data;
  }

  @override
  Map<String, dynamic>? toPutJson() {
    return null;
  }

  @override
  Future<FormData?>? toFormData() {
    return null;
  }
}

class NotificationBody extends BodyDataModel {
  String? title;
  String? description;
  String? photoUrl;
  String? userId;
  List<String>? products;
  List<String>? categories;
  List<String>? campaigns;
  String? storeId;

  NotificationBody({
    this.title,
    this.description,
    this.photoUrl,
    this.userId,
    this.products,
    this.categories,
    this.campaigns,
    this.storeId,
  });

  @override
  Map<String, dynamic>? toPostJson() {
    Map<String, dynamic> data = <String, dynamic>{};
    data["title"] = title;
    data["description"] = description;
    data["photoUrl"] = photoUrl;
    data["userId"] = userId;
    if (products != null) {
      data["products"] = products!.map((e) => e).toList();
    }

    if (categories != null) {
      data["categories"] = categories!.map((e) => e).toList();
    }
    if (campaigns != null) {
      data["campaigns"] = campaigns!.map((e) => e).toList();
    }
    data["storeId"] = storeId;
    return data;
  }

  @override
  Map<String, dynamic>? toPutJson() {
    Map<String, dynamic> data = <String, dynamic>{};
    data["title"] = title;
    data["description"] = description;
    data["photoUrl"] = photoUrl;
    data["userId"] = userId;
    if (products != null) {
      data["products"] = products!.map((e) => e).toList();
    }

    if (categories != null) {
      data["categories"] = categories!.map((e) => e).toList();
    }
    if (campaigns != null) {
      data["campaigns"] = campaigns!.map((e) => e).toList();
    }
    data["storeId"] = storeId;
    return data;
  }

  @override
  Future<FormData?>? toFormData() {
    return null;
  }
}

class OrderAssignBody extends BodyDataModel {
  List<String>? users;

  OrderAssignBody({
    this.users,
  });

  @override
  Map<String, dynamic>? toPostJson() {
    Map<String, dynamic> data = <String, dynamic>{};
    if (users != null) {
      data["users"] = users!.map((e) => e).toList();
    }
    return data;
  }

  @override
  Map<String, dynamic>? toPutJson() {
    Map<String, dynamic> data = <String, dynamic>{};
    if (users != null) {
      data["users"] = users!.map((e) => e).toList();
    }
    return data;
  }

  @override
  Future<FormData?>? toFormData() {
    return null;
  }
}
