import 'package:parodia_admin/core/enums.dart';

/// A base class for all query models.
///
/// This class provides a common structure for all query models, including a page number and an amount.
/// It also provides a method to convert the query model to a JSON object, which can be used when making HTTP requests.
///
/// All query models should inherit from this class and add any additional properties that they need.
class QueryModel {
  /// The page number for the query.
  int? page;

  /// The amount of items to be fetched in the query.
  int? amount;

  Status? status;

  /// Creates a new instance of [QueryModel].
  ///
  /// The [page] and [amount] parameters can be used to specify the page number and amount for the query.
  /// If they are not provided, they will default to `null`.
  QueryModel({
    this.page,
    this.amount,
    this.status,
  });

  /// Converts the [QueryModel] to a JSON object.
  ///
  /// This method can be used when making HTTP requests to convert the query model to a format that can be sent in the request.
  /// It only includes the [page] and [amount] properties if they are not `null`.
  Map<String, dynamic> toJson() {
    Map<String, dynamic> data = <String, dynamic>{};
    if (page != null) data["page"] = page;
    if (amount != null) data["amount"] = amount;
    if (status != null) data["status"] = status.toString().split('.').last;
    return data;
  }
}

class StoreQuery extends QueryModel {
  String? storeId;

  StoreQuery({
    super.page,
    super.amount,
    this.storeId,
  });

  @override
  Map<String, dynamic> toJson() {
    Map<String, dynamic> data = super.toJson();
    if (storeId != null) data["storeId"] = storeId;
    return data;
  }
}

class BrandQuery extends QueryModel {
  String? storeId;
  Sort? sort;

  BrandQuery({
    super.page,
    super.amount,
    this.storeId,
    this.sort,
  });

  @override
  Map<String, dynamic> toJson() {
    Map<String, dynamic> data = super.toJson();
    if (storeId != null) data["storeId"] = storeId;
    if (sort != null) data["sort"] = sort.toString().split('.').last;
    return data;
  }
}

class CampaignQuery extends QueryModel {
  String? storeId;
  CampaignType? type;
  CampaignSort? sort;

  CampaignQuery({
    super.page,
    super.amount,
    this.storeId,
    this.type,
    this.sort,
  });

  @override
  Map<String, dynamic> toJson() {
    Map<String, dynamic> data = super.toJson();
    if (storeId != null) data["storeId"] = storeId;
    if (type != null) data["type"] = type.toString().split('.').last;
    if (sort != null) data["sort"] = sort.toString().split('.').last;
    return data;
  }
}

class ProductCatalogQuery extends QueryModel {
  String? productId;
  String? catalogId;
  String? storeId;

  ProductCatalogQuery({
    super.page,
    super.amount,
    this.productId,
    this.catalogId,
    this.storeId,
  });

  @override
  Map<String, dynamic> toJson() {
    Map<String, dynamic> data = super.toJson();
    if (productId != null) data["productId"] = productId;
    if (catalogId != null) data["catalogId"] = catalogId;
    return data;
  }
}

class CategoryQuery extends QueryModel {
  CategorySort? sortBy;
  String? storeId;

  CategoryQuery({
    super.page,
    super.amount,
    this.sortBy,
    this.storeId,
  });

  @override
  Map<String, dynamic> toJson() {
    Map<String, dynamic> data = super.toJson();
    if (storeId != null) data["storeId"] = storeId;
    if (sortBy != null) data["sortBy"] = sortBy.toString().split('.').last;
    return data;
  }
}

class OrderQuery extends QueryModel {
  String? storeId;
  OrderSort? sort;
  OrderStatus? orderStatus;
  bool? isPaid;
  bool? isTest;

  OrderQuery({
    super.page,
    super.amount,
    this.storeId,
    this.sort,
    this.orderStatus,
    this.isPaid,
    this.isTest,
  });

  @override
  Map<String, dynamic> toJson() {
    Map<String, dynamic> data = super.toJson();
    if (storeId != null) data["storeId"] = storeId;
    if (sort != null) data["sort"] = sort.toString().split('.').last;
    if (orderStatus != null) {
      data["orderStatus"] = orderStatus.toString().split('.').last;
    }
    if (isPaid != null) data["isPaid"] = isPaid;
    if (isTest != null) data["isTest"] = isTest;
    return data;
  }
}

class PhotoQuery extends QueryModel {
  String? storeId;
  String? folderId;

  PhotoQuery({
    super.page,
    super.amount,
    this.storeId,
    this.folderId,
  });

  @override
  Map<String, dynamic> toJson() {
    Map<String, dynamic> data = super.toJson();
    if (storeId != null) data["storeId"] = storeId;
    if (folderId != null) data["folderId"] = folderId;
    return data;
  }
}

class ProductPriceAndRatingQuery extends QueryModel {
  String? storeId;
  String? productId;

  ProductPriceAndRatingQuery({
    super.page,
    super.amount,
    this.storeId,
    this.productId,
  });

  @override
  Map<String, dynamic> toJson() {
    Map<String, dynamic> data = super.toJson();
    if (storeId != null) data["storeId"] = storeId;
    if (productId != null) data["productId"] = productId;
    return data;
  }
}

class ProductQuery extends QueryModel {
  String? storeId;
  bool? grouped;
  String? search;
  ProductSort? sort;
  String? category;
  String? brand;
  List<String>? tags;
  List<String>? features;

  ProductQuery({
    super.page,
    super.amount,
    this.storeId,
    this.grouped,
    this.search,
    this.sort,
    this.category,
    this.brand,
    this.tags,
    this.features,
  });

  @override
  Map<String, dynamic> toJson() {
    Map<String, dynamic> data = super.toJson();
    if (storeId != null) data["storeId"] = storeId;
    if (grouped != null) data["grouped"] = grouped;
    if (search != null) data["search"] = search;
    if (sort != null) data["sort"] = sort.toString().split('.').last;
    if (category != null) data["category"] = category;
    if (brand != null) data["brand"] = brand;
    if (tags != null) data["tags"] = tags;
    if (features != null) data["features"] = features;
    return data;
  }
}

class ReportQuery extends QueryModel {
  String? storeId;
  ReportType? type;
  DateType? dateType;
  CurrencyType? currencyType;
  bool? withPrices;
  bool? isTest;
  String? startDate;
  String? endDate;

  ReportQuery({
    super.page,
    super.amount,
    this.storeId,
    this.type,
    this.dateType,
    this.currencyType,
    this.withPrices,
    this.isTest,
    this.startDate,
    this.endDate,
  });

  @override
  Map<String, dynamic> toJson() {
    Map<String, dynamic> data = super.toJson();
    if (storeId != null) data["storeId"] = storeId;
    if (type != null) data["type"] = type.toString().split('.').last;
    if (dateType != null) {
      data["dateType"] = dateType.toString().split('.').last;
    }
    if (currencyType != null) {
      data["currencyType"] = currencyType.toString().split('.').last;
    }
    if (withPrices != null) data["withPrices"] = withPrices;
    if (isTest != null) data["isTest"] = isTest;
    if (startDate != null) data["startDate"] = startDate;
    if (endDate != null) data["endDate"] = endDate;
    return data;
  }
}

class StockQuery extends QueryModel {
  String? storeId;
  String? productId;
  StockType? type;
  StockStatus? stockType;
  StockOrderStatus? stockStatus;

  StockQuery({
    super.page,
    super.amount,
    this.storeId,
    this.productId,
    this.type,
    this.stockType,
    this.stockStatus,
  });

  @override
  Map<String, dynamic> toJson() {
    Map<String, dynamic> data = super.toJson();
    if (type != null) data["type"] = type.toString().split('.').last;
    if (stockType != null) {
      if (stockType == StockStatus.stock_in) {
        data["stockType"] = "in";
      } else {
        data["stockType"] = stockType.toString().split('.').last;
      }
    }
    if (stockStatus != null) {
      data["stockStatus"] = stockStatus.toString().split('.').last;
    }
    if (storeId != null) data["storeId"] = storeId;
    if (productId != null) data["productId"] = productId;
    return data;
  }
}

class TicketQuery extends QueryModel {
  String? storeId;
  TicketFilter? filter;
  String? code;

  TicketQuery({
    super.page,
    super.amount,
    this.storeId,
    this.filter,
    this.code,
  });

  @override
  Map<String, dynamic> toJson() {
    Map<String, dynamic> data = super.toJson();
    if (filter != null) data["filter"] = filter.toString().split('.').last;
    if (code != null) data["code"] = code;
    if (storeId != null) data["storeId"] = storeId;
    return data;
  }
}

class StoreUserQuery extends QueryModel {
  String? storeId;
  StoreUserType? type;

  StoreUserQuery({
    super.page,
    super.amount,
    this.storeId,
    this.type,
  });

  @override
  Map<String, dynamic> toJson() {
    Map<String, dynamic> data = super.toJson();
    if (storeId != null) data["storeId"] = storeId;
    if (type != null) data["type"] = type.toString().split('.').last;
    return data;
  }
}

class VariantQuery extends QueryModel {
  String? storeId;
  String? groupId;

  VariantQuery({
    super.page,
    super.amount,
    this.storeId,
    this.groupId,
  });

  @override
  Map<String, dynamic> toJson() {
    Map<String, dynamic> data = super.toJson();
    if (groupId != null) data["groupId"] = groupId;
    if (storeId != null) data["storeId"] = storeId;
    return data;
  }
}

class StoreParentQuery extends QueryModel {
  String? storeId;
  String? parentId;

  StoreParentQuery({
    super.page,
    super.amount,
    this.storeId,
    this.parentId,
  });

  @override
  Map<String, dynamic> toJson() {
    Map<String, dynamic> data = super.toJson();
    if (parentId != null) data["parentId"] = parentId;
    if (storeId != null) data["storeId"] = storeId;
    return data;
  }
}

class SuggestionQuery extends QueryModel {
  String? storeId;
  bool? solved;

  SuggestionQuery({
    super.page,
    super.amount,
    this.storeId,
    this.solved,
  });

  @override
  Map<String, dynamic> toJson() {
    Map<String, dynamic> data = super.toJson();
    if (storeId != null) data["storeId"] = storeId;
    if (solved != null) data["solved"] = solved;
    return data;
  }
}
