import 'package:parodia_admin/core/model/validation_error.model.dart';

/// A base class for all response models.
///
/// This class provides a common structure for all response models, including a status code, message, data, affected rows, extra data, and a list of required validation errors.
/// It also provides methods to convert the response model to and from a JSON object, and a method to copy the response model.
///
/// All response models should inherit from this class and add any additional properties that they need.
class ResponseModel {
  /// The status code of the response.
  int? statusCode;

  /// The message of the response.
  String? message;

  /// The main data of the response.
  dynamic data;

  /// The number of rows affected by the request.
  int? affectedRows;

  /// Any extra data included in the response.
  dynamic extra;

  /// A list of validation errors that were required for the request.
  List<ValidationErrorModel>? required;

  /// Creates a new instance of [ResponseModel].
  ///
  /// The parameters can be used to specify the initial values for the response model's properties.
  ResponseModel({
    this.statusCode,
    this.message,
    this.data,
    this.affectedRows,
    this.extra,
    this.required,
  });

  /// Creates a new instance of [ResponseModel] from a JSON object.
  ///
  /// This method extracts the properties from the [json] parameter and uses them to initialize a new instance of [ResponseModel].
  ResponseModel.fromJson(Map<String, dynamic> json) {
    statusCode = json['statusCode'];
    message = json['message'];
    data = json['data'];
    affectedRows = json['affectedRows'];
    extra = json['extra'];
    if (json['required'] != null) {
      required = <ValidationErrorModel>[];
      json['required'].forEach((v) {
        required!.add(ValidationErrorModel.fromJson(v));
      });
    }
  }

  /// Converts the [ResponseModel] to a JSON object.
  ///
  /// This method can be used when making HTTP requests to convert the response model to a format that can be sent in the request.
  Map<String, dynamic> toJson() {
    Map<String, dynamic> data = <String, dynamic>{};
    data["statusCode"] = statusCode;
    data["affectedRows"] = affectedRows;
    data["message"] = message;
    data["data"] = data;
    data["extra"] = extra;
    if (required != null) {
      data["required"] = required!.map((e) => e.toJson()).toList();
    }
    return data;
  }

  /// Creates a copy of the [ResponseModel] with the specified values.
  ///
  /// This method can be used to create a new instance of [ResponseModel] that has the same values as the current instance, but with some values replaced by the values specified in the parameters.
  copyWith({
    int? statusCode,
    String? message,
    dynamic data,
    int? affectedRows,
    dynamic extra,
    List<ValidationErrorModel>? required,
  }) {
    return ResponseModel(
      statusCode: statusCode ?? this.statusCode,
      message: message ?? this.message,
      data: data ?? this.data,
      affectedRows: affectedRows ?? this.affectedRows,
      extra: extra ?? this.extra,
      required: required ?? this.required,
    );
  }
}
