import 'package:flutter/material.dart';
import 'package:parodia_admin/core/parodia.dart';
import 'package:provider/provider.dart';
import 'package:provider/single_child_widget.dart';

/// A class for initializing and managing the Parodia provider.
///
/// This class extends [StatelessWidget] and provides methods to initialize the Parodia service and build the widget tree.
/// It uses the [MultiProvider] widget to provide multiple providers to the widget tree.
class ParodiaProvider extends StatelessWidget {
  /// The base URL for the Parodia service.
  final String baseUrl;

  /// The store key for the Parodia service.
  final String storeKey;

  /// The authentication token for the Parodia service.
  final String? authToken;

  /// The list of additional providers for the Parodia service.
  final List<SingleChildWidget>? providers;

  /// The child widget to be built by the [ParodiaProvider].
  final Widget? child;

  /// The builder function to build the widget tree.
  final Widget Function(BuildContext, Widget?)? builder;

  /// Creates a new instance of [ParodiaProvider].
  ///
  /// The [baseUrl] and [storeKey] parameters are required to initialize the Parodia service.
  /// The [authToken], [providers], [builder], and [child] parameters are optional.
  const ParodiaProvider({
    Key? key,
    required this.baseUrl,
    required this.storeKey,
    this.authToken,
    this.providers,
    this.builder,
    this.child,
  }) : super(key: key);

  /// Builds the widget tree for the [ParodiaProvider].
  ///
  /// This method initializes the Parodia service and then uses the [MultiProvider] widget to provide multiple providers to the widget tree.
  @override
  Widget build(BuildContext context) {
    Parodia.instance.init(
      baseUrl: baseUrl,
      storeKey: storeKey,
      authToken: authToken,
    );
    return MultiProvider(
      providers: [
        ...Parodia.instance.providers,
        ...?providers,
      ],
      builder: builder,
      child: child,
    );
  }
}
